# -*- coding: utf-8 -*-

import base64
import logging
import subprocess
import tempfile

from openerp.exceptions import Warning as UserError
from openerp.tools.translate import _

import M2Crypto as m2


_logger = logging.getLogger(__name__)


class PkiUtils(object):
    """Util class to get PEM from DER cert & key """

    def __init__(self, cert, key, passwd):
        self.key = key
        self.passwd = passwd
        self.key_pem = self._get_key_pem()
        self.cert_der = self._get_cer_der(cert)

    @classmethod
    def _get_cer_der(cls, cert):
        "get cer in DER encode"
        cer_der_decode = base64.decodestring(cert)
        try:
            cert_der = m2.X509.load_cert_der_string(cer_der_decode)
        except m2.X509.X509Error:
            _logger.warning(m2.X509.X509Error.message)
            raise UserError(
                _(
                    "You are trying to upload an known format file."
                    " File must be encoded as DER. "
                    "For further information review log"
                )
            )
        else:
            return cert_der

    def _get_key_pem(self):
        """Get key in pem encode from der encode """
        private_key_file_pem = tempfile.NamedTemporaryFile()
        with tempfile.NamedTemporaryFile() as private_key_file_der:
            private_key_file_der.write(base64.decodestring(self.key))
            private_key_file_der.flush()
            subprocess.call(
                "openssl pkcs8 -inform DER -outform PEM"
                ' -in "%s" -passin pass:%s -out %s'
                % (
                    private_key_file_der.name,
                    self.passwd,
                    private_key_file_pem.name,
                ),
                shell=True,
            )
            key_pem = private_key_file_pem.read()
        private_key_file_pem.close()
        return key_pem

    def get_key_pem(self):
        return self.key_pem

    def get_cert_pem(self):
        return self.cert_der.as_pem()

    def _get_module_from_key(self):
        """Helper file to read module from private key"""
        file_module = tempfile.NamedTemporaryFile()
        with tempfile.NamedTemporaryFile() as key_pem_file:
            key_pem_file.write(self.key_pem)
            key_pem_file.flush()
            cmd = 'openssl rsa -noout -modulus -in "%s" -out %s' % (
                key_pem_file.name,
                file_module.name,
            )
            subprocess.call(cmd, shell=True)
            key_module = file_module.read().replace("Modulus=", "")
        file_module.close()
        return key_module

    def validate_cert_key(self):
        """Compare modules to know if certificate and key match"""
        cert_module = self.cert_der.get_pubkey().get_modulus()
        key_module = self._get_module_from_key()
        if cert_module.strip() != key_module.strip():
            raise UserError(
                _(
                    "The certificate and private key that you are loading "
                    "do not match"
                )
            )

    def get_cer_der(self):
        return self.cert_der
