# -*- coding: utf-8 -*-

from openupgradelib import openupgrade as tools

from openerp import SUPERUSER_ID


@tools.migrate(use_env=True, uid=SUPERUSER_ID)
def migrate(env, installed_version):
    # Figure if column type_attachment exists on ir_attachment_facturae_mx
    # table, if not exist means migration is from pre 1.3.0 version and need
    # a special treatment during migration
    type_attachment_exists = tools.column_exists(
        env.cr, "ir_attachment_facturae_mx", "type_attachment"
    )
    # Some states where deprecated on ir.attachment.facturae.mx object
    # search for registers still using deprecated states and move to the
    # corresponding new one

    # Delete wrong records on ir_attachment_facturae_mx table
    tools.logged_query(
        env.cr,
        """DELETE FROM ir_attachment_facturae_mx
        WHERE state='confirmed' OR (state='cancel' AND file_xml_sign IS NULL)
        """,
    )
    # create new columns on ir_attachment_facturae_mx table
    tools.logged_query(
        env.cr,
        """ALTER TABLE ir_attachment_facturae_mx
ADD COLUMN config_id INTEGER,
ADD COLUMN uuid CHARACTER varying,
ADD COLUMN cfdi_fecha_cancelado TIMESTAMP WITHOUT TIME ZONE,
ADD COLUMN cfdi_fecha_timbrado TIMESTAMP WITHOUT TIME ZONE,
ADD COLUMN fecha TIMESTAMP WITHOUT TIME ZONE
        """,
    )
    # printable, sent_customer >> done
    tools.logged_query(
        env.cr,
        """UPDATE ir_attachment_facturae_mx
        SET state='done' WHERE state='printable' OR state='sent_customer'
        """,
    )

    # security measure -> remove duplicated rows
    if type_attachment_exists:
        tools.logged_query(
            env.cr,
            """DELETE FROM ir_attachment_facturae_mx
                WHERE id IN (
                    SELECT id
                    FROM (
                        SELECT id, ROW_NUMBER()
                        OVER (
                            partition BY res_id, company_id, type_attachment,
                            state
                            ORDER BY id
                        ) AS rnum FROM ir_attachment_facturae_mx
                    ) t
                    WHERE t.rnum > 1
                );""",
        )

    # To avoid slow computation of uuid field for ir.attachment.facturae.mx
    # create uuid field and copy uuids from account.invoice objects
    query = """UPDATE ir_attachment_facturae_mx AS cfdi
    SET uuid=invoice.cfdi_folio_fiscal,
    cfdi_fecha_timbrado=invoice.cfdi_fecha_timbrado,
    fecha=invoice.cfdi_fecha_timbrado,
    cfdi_fecha_cancelado=invoice.cfdi_fecha_cancelacion
    FROM account_invoice AS invoice
    WHERE cfdi.state='done'
    AND cfdi.res_id=invoice.id AND invoice.type in ('out_invoice', 'out_refund')
    """
    if type_attachment_exists:
        query += "AND cfdi.type_attachment='account.invoice'"
    tools.logged_query(env.cr, query)
