# -*- coding: utf-8 -*-

from openerp import fields, models


class IrAttachmentFacturaeMx(models.Model):
    """Model used to hold configuration for different type of CFDI documents
    that can be generated on the system.
    """

    _name = "ir.attachment.facturae.mx.config"
    _description = __doc__

    _sql_constraints = [
        (
            "unique_model",
            "unique (model, active, version)",
            "It is no possible to define more than one active template for a "
            "single model and version!",
        )
    ]

    model = fields.Char(required=True, readonly=True, help="This is the object model")
    active = fields.Boolean(default=True)
    version = fields.Selection(
        [("3.2", "3.2"), ("3.3", "3.3")],
        required=True,
        help="Specification Version used to create this CFDI",
    )
    template_xml_sign = fields.Char(
        "XML report", required=True, help="Name of XML template when signing"
    )
    template_pdf_sign = fields.Char(
        "PDF report", required=True, help="Name of PDF template when signing"
    )
    email_template_id = fields.Many2one(
        "email.template",
        help="Name of email template to use when sending email to partner",
    )
    template_xml_cancel = fields.Char(
        "Cancelation XML report",
        required=False,
        help="Name of XML template when canceling",
    )
    template_pdf_cancel = fields.Char(
        required=False, help="Name of PDF template when canceling"
    )
