# -*- coding: utf-8 -*-

import logging
import time

from openerp import api, fields, models


_logger = logging.getLogger(__name__)


class ResCompany(models.Model):
    _inherit = "res.company"

    pac_ids = fields.Many2many("params.pac", string="Pacs")
    certificate_ids = fields.One2many(
        "res.company.facturae.certificate",
        "company_id",
        "Certificates",
        help="Certificates configurated in this company",
    )
    certificate_id = fields.Many2one(
        comodel_name="res.company.facturae.certificate",
        string="Certificate Current",
        compute="_compute_certificate_id",
        help="Serial Number of the  certificate active and inside of "
        "dates in this company",
    )
    fiel_id = fields.Many2one(
        "res.company.facturae.fiel",
        string="Fiel Current",
        compute="_compute_fiel_id",
        help="Serial number of the fiel active and inside of dates in"
        "this company",
    )
    fiel_ids = fields.One2many(
        "res.company.facturae.fiel",
        "company_id",
        help="Fiels configurated in this company",
    )

    @api.multi
    def _compute_certificate_id(self):
        for company in self:
            cert_obj = self.env["res.company.facturae.certificate"]
            date = time.strftime("%Y-%m-%d")
            domain = [
                ("company_id", "=", company.id),
                ("date_start", "<=", date),
                ("date_end", ">=", date),
                ("active", "=", True),
            ]
            company.certificate_id = cert_obj.search(domain, limit=1)

    @api.multi
    def _compute_fiel_id(self):
        for company in self:
            fiel_obj = self.env["res.company.facturae.fiel"]
            date = time.strftime("%Y-%m-%d")
            domain = [
                ("company_id", "=", company.id),
                ("date_start", "<=", date),
                ("date_end", ">=", date),
                ("active", "=", True),
            ]
            company.fiel_id = fiel_obj.search(domain, limit=1)

    @api.model
    def _get_info(self, vals):
        """Function uses to get such values as: end certificate date,
        start certificate date and serial number """
        cer_der = vals._load_certificate_and_private_key(vals)
        # Get serial number from certificate
        serial_number = cer_der.get_serial_number()
        # but what really they are a long type
        # But they are ASCII characters
        serial_number = "{0:x}".format(serial_number).decode("hex")
        # Time in datetime object
        date_start = cer_der.get_not_before().get_datetime()
        date_end = cer_der.get_not_after().get_datetime()
        # Transform into string
        date_start = date_start.strftime("%Y-%m-%d")
        date_end = date_end.strftime("%Y-%m-%d")
        # Write values
        vals.write(
            {
                "date_start": date_start,
                "date_end": date_end,
                "serial_number": serial_number,
            }
        )
