# -*- coding: utf-8 -*-

import logging

from openerp import api, fields, models

from ..lib.utils import PkiUtils


_logger = logging.getLogger(__name__)


class ResCompanyFacturaeCertificate(models.Model):
    _name = "res.company.facturae.certificate"

    _rec_name = "serial_number"

    @api.model
    def _default_company_id(self):
        res_company_obj = self.env["res.company"]
        company = res_company_obj._company_default_get(
            "res.company.facturae.certificate"
        )
        return company

    company_id = fields.Many2one(
        "res.company",
        "Company",
        required=True,
        default=lambda self: self._default_company_id(),
        help="Company where you add this certificate",
    )
    certificate_file = fields.Binary(
        filters="*.cer,*.certificate,*.cert",
        required=True,
        help="This file .cer is proportionate by the SAT",
    )
    certificate_key_file = fields.Binary(
        filters="*.key",
        required=True,
        help="This file .key is proportionate by the SAT",
    )
    certificate_password = fields.Char(
        size=64,
        invisible=False,
        required=True,
        help="This password is proportionate by the SAT",
    )
    certificate_file_pem = fields.Binary(
        filters="*.pem,*.cer,*.certificate,*.cert",
        help="This file is generated with the file.cer",
    )
    certificate_key_file_pem = fields.Binary(
        filters="*.pem,*.key", help="This file is generated with the file.key"
    )
    date_start = fields.Date(help="Date start the certificate before the SAT")
    date_end = fields.Date(help="Date end of validity of the certificate")
    serial_number = fields.Char(help="Number of serie of the certificate")
    fname_xslt = fields.Char(
        "File XML Parser (.xslt)", help="Folder in server with XSLT file"
    )
    active = fields.Boolean(
        default=True, help="Indicate if this certificate is active"
    )

    @api.model
    @api.returns("self", lambda value: value.id)
    def create(self, vals):
        res = super(ResCompanyFacturaeCertificate, self).create(vals)
        company = self.env["res.company"]
        company._get_info(res)
        return res

    def _load_certificate_and_private_key(self, vals):
        """Verify certificate and key files to be loaded are right
        If they are right return files in der format to be parsed
        """
        pkutils = PkiUtils(
            vals.certificate_file, vals.certificate_key_file, vals.certificate_password
        )
        pkutils.validate_cert_key()
        cer_pem = pkutils.get_cert_pem()
        key_pem = pkutils.get_key_pem()
        # Save certificate as PEM and KEY as PEM too
        vals.write(
            {"certificate_file_pem": cer_pem, "certificate_key_file_pem": key_pem}
        )
        return pkutils.get_cer_der()
