# -*- coding: utf-8 -*-

import base64

from openerp.tests.common import TransactionCase
from openerp.tools import file_open


class TestCertificados(TransactionCase):

    def _load_files(self, name="EKU9003173C9"):
        f = file_open(
            "l10n_mx_ir_attachment_facturae/examples/{name}.cer".format(name=name),
            "rb"
        )
        certificate_file = base64.b64encode(f.read())
        f.close()
        f = file_open(
            "l10n_mx_ir_attachment_facturae/examples/{name}.key".format(name=name),
            "rb"
        )
        certificate_key = base64.b64encode(f.read())
        f.close()
        return certificate_file, certificate_key

    def test_cfdi_extract_information(self):
        """Test if correct extract information when creating a new
        res.company.facturae.certificate"""
        cer, key = self._load_files()
        certificate = self.env["res.company.facturae.certificate"].create(
            {
                "certificate_file": cer,
                "certificate_key_file": key,
                "certificate_password": "12345678a",
            }
        )
        self.assertEqual(
            certificate.serial_number,
            "30001000000400002434",
            "Woring serial number extracted from files!",
        )
        self.assertEqual(
            certificate.date_start,
            "2019-06-17",
            "Woring date start extracted from files!",
        )
        self.assertEqual(
            certificate.date_end,
            "2023-06-17",
            "Woring date end extracted from files!",
        )

    def test_two_certificates(self):
        """Test if creating two certificates works ok"""

        cer, key = self._load_files()
        certificate = self.env["res.company.facturae.certificate"].create(
            {
                "certificate_file": cer,
                "certificate_key_file": key,
                "certificate_password": "12345678a",
            }
        )
        self.assertEqual(
            certificate.serial_number,
            "30001000000400002434",
            "Woring serial number extracted from files!",
        )
        self.assertEqual(
            certificate.date_start,
            "2019-06-17",
            "Woring date start extracted from files!",
        )
        self.assertEqual(
            certificate.date_end,
            "2023-06-17",
            "Woring date end extracted from files!",
        )

        # Create new certificate
        cer, key = self._load_files(name="H&E951128469")
        certificate = self.env["res.company.facturae.certificate"].create(
            {
                "certificate_file": cer,
                "certificate_key_file": key,
                "certificate_password": "12345678a",
            }
        )
        self.assertEqual(
            certificate.serial_number,
            "30001000000400002463",
            "Woring serial number extracted from files!",
        )
        self.assertEqual(
            certificate.date_start,
            "2019-06-20",
            "Woring date start extracted from files!",
        )
        self.assertEqual(
            certificate.date_end,
            "2023-06-20",
            "Woring date end extracted from files!",
        )
