# -*- coding: utf-8 -*-

import pkg_resources

from openerp.tests.common import TransactionCase


original_chain_to_test = """
||3.3|VEN 2020|0045|2020-12-04T20:16:16|99||18.00|0.00|MXN|20.88|I|
PUE|72180|EKU9003173C9|YourCompany|601|AAÑ070522RLA|Axelor|G03|
01010101|ADPT|1.000000|H87|[ADPT] Adaptador USB|18.0|18.00|0.00|
18.00|002|Tasa|0.160000|2.88|002|Tasa|0.160000|2.88|2.88||
"""

certificate_string_to_test = """
MIIEYTCCA0mgAwIBAgIUMjAwMDEwMDAwMDAyMDAwMDE0MzcwDQYJKoZIhvcNA
QEFBQAwggFcMRowGAYDVQQDDBFBLkMuIDIgZGUgcHJ1ZWJhczEvMC0GA1UECg
wmU2VydmljaW8gZGUgQWRtaW5pc3RyYWNpw7NuIFRyaWJ1dGFyaWExODA2BgN
VBAsML0FkbWluaXN0cmFjacOzbiBkZSBTZWd1cmlkYWQgZGUgbGEgSW5mb3Jt
YWNpw7NuMSkwJwYJKoZIhvcNAQkBFhphc2lzbmV0QHBydWViYXMuc2F0LmdvY
i5teDEmMCQGA1UECQwdQXYuIEhpZGFsZ28gNzcsIENvbC4gR3VlcnJlcm8xDj
AMBgNVBBEMBTA2MzAwMQswCQYDVQQGEwJNWDEZMBcGA1UECAwQRGlzdHJpdG8
gRmVkZXJhbDESMBAGA1UEBwwJQ295b2Fjw6FuMTQwMgYJKoZIhvcNAQkCDCVS
ZXNwb25zYWJsZTogQXJhY2VsaSBHYW5kYXJhIEJhdXRpc3RhMB4XDTEzMDUwN
zE3NDAwN1oXDTE3MDUwNzE3NDAwN1owgdsxKTAnBgNVBAMTIEFDQ0VNIFNFUl
ZJQ0lPUyBFTVBSRVNBUklBTEVTIFNDMSkwJwYDVQQpEyBBQ0NFTSBTRVJWSUN
JT1MgRU1QUkVTQVJJQUxFUyBTQzEpMCcGA1UEChMgQUNDRU0gU0VSVklDSU9T
IEVNUFJFU0FSSUFMRVMgU0MxJTAjBgNVBC0THEFBQTAxMDEwMUFBQSAvIEhFR
1Q3NjEwMDM0UzIxHjAcBgNVBAUTFSAvIEhFR1Q3NjEwMDNNREZOU1IwODERMA
8GA1UECxMIcHJvZHVjdG8wgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAJ4
COACtDKzGDv9V0W3LN508rc2eICyi5g3SsGrJE1Z49cJngFuR6DgXWUO85Tu/
NG3r4aXOvZs6bGG6dEG7Dcb1aC5dkF+yI6PlHKsiTk5ntJ/aETA/rckVyb9cm
eCh4Mqfo0OGxMmsEzxUl7qm3onv2ldmk6pJmSIQGTHzMZYbAgMBAAGjHTAbMA
wGA1UdEwEB/wQCMAAwCwYDVR0PBAQDAgbAMA0GCSqGSIb3DQEBBQUAA4IBAQD
WG+0Soy5vxmclDKOrvyHjAbQGk4BPynbCjOWeFZ5s7HOpw6TSSe56R33n/ZO2
CvC4/ICwrNgl+drycfU5ZA5d3TflsO9jgTBsDSq5LDJ9h062IadD8o4Mg5BTb
1C2FAiMD16DHwxVgYZvk2wmXQ0mrxLoMu2wiBpbvwoXJlOiociUxXQtlukmjz
meTovu1XTDd50gbztYBHIdjOpAjXmmytwrwNGmWG+D3Jfjhtydy1GSOTtN+hG
ElYg71vLNFekJyEkc4pkWORG0xmwRf/rnoSFjHyk/5duRUth6GEvQEfwAlZIn
7rsttGzubLO9QY5BM6P3km98mPt7NnV/SSxo
"""


sello_to_test = """
nOUBld3LOncqfivgmaoF1BrQUx8A82uGcx8uPYrA1QkrYIfE2VHuSnl4lvIYJA+Ig
FN/LhiPxIGrLQfCcLUUeccPBykijGoGNhtVDA6ARuhHIJv/PoM3f65HU+QFcIeqMD
pyGwp2mOrjNHNzsjFRftv74SuFD7/tnNxvqmgigBc=
"""


class CFDI(TransactionCase):
    """Unit test for CFDI functions"""

    def setUp(self):
        super(CFDI, self).setUp()
        self.cfdi = self.registry("ir.attachment.facturae.mx")

    def test_xml2cad_orig(self):
        """Test original chain creation using 3.2v"""
        # Get paths to necessary test
        path_xml = pkg_resources.resource_filename(
            "openerp.addons.l10n_mx_ir_attachment_facturae",
            "tests/unit/cfdi_33_before_sign.xml",
        )
        path_xlst = pkg_resources.resource_filename(
            "openerp.addons.l10n_mx_ir_attachment_facturae",
            "cfd/cadenaoriginal_3_3.xslt",
        )
        with open(path_xml, "rb") as file:
            xml = file.read()
        original_chain = self.cfdi._xml2cad_orig(xml, path_xlst)
        self.assertEqual(
            original_chain,
            original_chain_to_test.replace("\n", "").strip(),
            "Original chain creation went wrong",
        )

    def test_get_certificate_str(self):
        """Test getting certificate string"""
        # Get paths to necessary test
        certifcate_pem_path = pkg_resources.resource_filename(
            "openerp.addons.l10n_mx_ir_attachment_facturae", "tests/unit/certifcate.pem"
        )
        with open(certifcate_pem_path, "rb") as file:
            certifcate_pem = file.read()
        certificate_string = self.cfdi._get_certificate_str(certifcate_pem)
        self.assertEqual(
            certificate_string,
            certificate_string_to_test.replace("\n", "").strip(),
            "Certificate string went wrong",
        )

    def test_get_sello_sha256(self):
        """Test getting sello using sha1"""
        # Get paths to necessary test
        key_pem_path = pkg_resources.resource_filename(
            "openerp.addons.l10n_mx_ir_attachment_facturae", "tests/unit/key.pem"
        )
        with open(key_pem_path, "rb") as file:
            key_pem = file.read()
        sello = self.cfdi._get_sello(key_pem, original_chain_to_test, "sha256")
        self.assertEqual(
            sello, sello_to_test.replace("\n", "").strip(), "Sello went wrong"
        )
