# -*- coding: utf-8 -*-

# List use to know if error must be handled with ValidationError exception
validation_error_list = [
    "301",
    "302",
    "303",
    "304",
    "305",
    "306",
    "308",
    "401",
    "402",
    "403",
    "611",
    "612",
]

# List use to know if error must be handled with CancelError exception
cancel_error_list = [
    "611",
    "621",
    "702",
    "1701",
    "1702",
    "1703",
    "1704",
    "1710",
    "1711",
    "1712",
    "1713",
    "1803",
    "203",
    "204",
    "205",
    "402",
]


class PacError(Exception):
    """General class for errors define in this module"""

    pass


class PrimarySectorError(PacError):
    """
    This error should handle errors as described in SAT table named
    Codigos de estatus para el sector primario as shown forward.

    311 - No autorizado.
    312 - Error al consultar al SAT
    """

    def __init__(self, code, message=""):
        self.code = code
        self.message = message
        super(PrimarySectorError, self).__init__()

    def __repr__(self):
        if self.code == "311":
            message = "No autorizado"
        elif self.code == "312":
            message = "Error al consultar al SAT"
        message = "Error code: {code} \n {message}".format(
            code=self.code, message=message
        )
        return message


class ValidationError(PrimarySectorError):
    """
    This error should handle errors as described in SAT table named
    Validacion CFDI as shown forward.

    Handle error here only must be errors which user could fix himself only
    with configuration, for example a missing data or a typo.

    Next values are the same inside validate_error_list

    501 - Usuario o contraseña invalidos.

    301 - El CFDI no tiene una estructura XML correcta.
    302 - El sello del emisor no es válido.
    303 - El Certificado de Sello Digital no corresponde al contribuyente emisor.
    304 - El certificado se encuentra revocado o caduco.
    305 - La fecha del CFDI está fuera del rango de la validez del certificado.
    306 - El certificado usado para generar el sello digital no es un Certificado
          de Sello Digital.
    308 - El certificado utilizado para generar el sello digital no ha sido emitido
          por el SAT.
    401 - La fecha del comprobante está fuera del rango de timbrado permitido.
    402 - El contribuyente no se encuentra dentro del régimen fiscal para emitir CFDI.
    403 - La fecha de emisión del CFDI no puede ser anterior al 1 de enero de 2011.
    611 - Los parámetros recibidos están incompletos o no se encuentran donde se
          esperarían.
    612 - El archivo XML o alguno de sus atributos está malformado.
    621 - Datos no válidos. La estructura de los datos contenidos en el XML no es
          válida para CFDI.
    """

    def __repr__(self):
        message = "Error code: {code} \n {message}".format(
            code=self.code, message=self.message.encode("UTF-8")
        )
        return message


class CancelError(ValidationError):
    """
    This error should handle errors as described in SAT table named
    Cancelacion CFDI as shown forward.

    Handle error here only must be errors which user could fix himself only
    with configuration, for example a missing data or a typo.

    Next values are the same inside cancel_error_list

    611 - No se han proporcionado UUIDs a cancelar.
    621 - Formato o estructura de argumento no válida para el tipo de dato.
    1701 - La llave privada y la llave pública del CSD no corresponden.
    1702 - La llave privada de la contraseña es incorrecta.
    1703 - La llave privada no cumple con la estructura esperada.
    1704 - La llave Privada no es una llave RSA.
    1710 - La estructura del certificado no cumple con la estructura X509 esperada.
    1711 - El certificado no esá vigente todavía.
    1712 - El certificado ha expirado.
    1713 - La llave pública contenida en el certificado no es una llave RSA.
    1803 - El dato no es un UUID válido.

    203 - UUID no corresponde al emisor.
    204 - El CFDI no aplica para cancelación.
    205 - El UUID no existe o no ha sido procesado por el SAT.
    402 - El Contribuyente no se encuentra el la LCO o la validez de obligaciones se
          reporta como negativa.
    """

    pass
