# -*- coding: utf-8 -*-
# pylint: skip-file
# flake8: noqa

import base64
import logging

# Librerías
import os
import string

# caracteres de escape
from xml.sax.saxutils import escape

from suds.client import Client

from openerp import api, fields, models, tools
from openerp.osv import osv
from openerp.tools.translate import _


logger = logging.getLogger(__name__)


class params_pac(models.Model):
    _inherit = "params.pac"

    pac_type = fields.Selection(
        selection_add=[("cfdi32_4g", _("CFDI 3.2 4G Factor"))]
    )

    def get_driver_fc_sign(self):
        """
        Implement driver for sign invoices
        """
        factura_mx_type__fc = super(params_pac, self).get_driver_fc_sign()
        if factura_mx_type__fc == None:
            factura_mx_type__fc = {}
        factura_mx_type__fc.update({"cfdi32_4g": self._upload_ws_file_4g})
        return factura_mx_type__fc

    def get_driver_fc_cancel(self):
        """
        Implement driver for cancel invoices
        """
        factura_mx_type__fc = super(params_pac, self).get_driver_fc_cancel()
        if factura_mx_type__fc == None:
            factura_mx_type__fc = {}
        factura_mx_type__fc.update({"cfdi32_4g": self._cancel_ws_file_4g})
        return factura_mx_type__fc

    def _get_ws(self, wsparam):
        WS = ""
        # Search char "," for addons_path, now is multi-path
        all_paths = tools.config["addons_path"].split(",")
        for my_path in all_paths:
            if os.path.isdir(os.path.join(my_path, "l10n_mx_facturae_pac_4g", "ws")):
                # If dir is in path, save it on real_path
                WS = (
                    my_path
                    and os.path.join(my_path, "l10n_mx_facturae_pac_4g", "ws", wsparam)
                    or ""
                )
                WS = "file:///" + WS
        return WS

    def _upload_ws_file_4g(self, cr, uid, ids, fdata=None, context={}):
        """
        @params
            invoice_id: Id from invoice to validate
            fdata : File.xml base64 codificated
        """
        if context is None:
            context = {}
        pac_params_obj = self.pool.get("params.pac")
        for attachment in self.browse(cr, uid, ids, context=context):
            invoice = attachment.invoice_id
            f = False
            timbre = False
            msg = ""
            pac_ids = pac_params_obj.search(
                cr,
                uid,
                [
                    ("method_type", "=", "pac_4g_firmar"),
                    ("company_id", "=", invoice.company_emitter_id.id),
                    ("active", "=", True),
                ],
                limit=1,
                context=context,
            )

            if not pac_ids:
                raise osv.except_osv(
                    _("Warning"),
                    _(
                        "Not found information from web services of PAC"
                        "Verify that the configuration of PAC is correct"
                    ),
                )

            # Get pac params for use on this invoice
            pac = pac_params_obj.browse(cr, uid, pac_ids, context)[0]

            if pac.user == "VE223.test_ws":
                msg += _("WARNING, SIGNED IN TEST 4G FACTOR!!!!\n\n")
            try:
                WSDL = self._get_ws(pac.url_webservice)
                client = Client(WSDL)
                # Crea sesión
                try:
                    session = client.service.openSession(pac.user, pac.password)
                except Exception as e:
                    # Error al establecer comunicación con el PAC
                    logger.debug(str(e))
                    raise osv.except_osv(
                        _("Error"),
                        _(u"PAC communication failed (openSession). \n%s") % str(e),
                    )
                # Obtén token de sesión
                if session.ok:
                    # Crea objeto CFD a timbrar
                    cfd = client.factory.create("ns0:cfdRecord")
                    # Establece token de sesión
                    cfd.token = session.token
                    # Asigna XML
                    fdatadecode = base64.b64decode(fdata)
                    fdatadecode = fdatadecode.decode("UTF-8")
                    fdatadecode = escape(fdatadecode)
                    fdatadecode = fdatadecode.replace("'", "&#39;")
                    cfd.xml = fdatadecode
                    # Timbra CFD
                    cfdi = None
                    try:
                        cfdi = client.service.createCFDI(cfd)
                    except Exception as e:
                        raise osv.except_osv(
                            _("Error"),
                            _(u"Create CFDI communication failed. \n%s") % str(e),
                        )
                    # Verifica timbrado
                    if cfdi.ok:
                        # Codifica el XML para almacenarlo
                        f = base64.encodestring(cfdi.xml.encode("UTF-8") or "")
                        timbre = cfdi.xml
                        msg += string.join(
                            [_("successful process"), ". Folio Fiscal ", cfdi.uuid, "."]
                        )
                        # Cierra token
                        client.service.closeSession(cfd.token)
                    else:
                        # Error al generar CFDI
                        raise osv.except_osv(
                            _("Error"),
                            _(u"could not stamp CFDI, code: %d") % cfdi.errorCode,
                        )
                else:
                    raise osv.except_osv(
                        _("Error"),
                        _("Error, could not create a session, code: %d")
                        % str(session.errorCode),
                    )
            except Exception as err:
                raise err
        return {"file": f, "msg": msg, "cfdi_xml": timbre}

    def _cancel_ws_file_4g(self, cr, uid, ids, context=None):
        """
        Cancel xml file onto PAC webservice
        @params
            id:s Id from invoice to validate
        """
        import time

        if context is None:
            context = {}
        pac_params_obj = self.pool.get("params.pac")
        for attachment in self.browse(cr, uid, ids, context=context):
            invoice = attachment.invoice_id
            msg = ""
            cod_status = ""

            # Get data from PAC configuration
            pac_ids = pac_params_obj.search(
                cr,
                uid,
                [
                    ("method_type", "=", "pac_4g_cancelar"),
                    ("company_id", "=", invoice.company_emitter_id.id),
                    ("active", "=", True),
                ],
                limit=1,
                context=context,
            )

            if not pac_ids:
                raise osv.except_osv(
                    _("Warning"),
                    _(
                        "Not found information from web services of PAC, "
                        "verify that the configuration of PAC is correct"
                    ),
                )

            pac = pac_params_obj.browse(cr, uid, pac_ids, context)[0]

            # Send file to PAC for cancelation
            if pac.user == "VE223.test_ws":
                msg += _(u"WARNING, CANCEL IN TEST 4G FACTOR!!!!\n\n")

            try:
                WSDL = self._get_ws(pac.url_webservice)
                client = Client(WSDL)
                # Crea sesión
                session = client.service.openSession(pac.user, pac.password)
                # Obtén token de sesión
                if session.ok:
                    # Crea objeto
                    request = client.factory.create("ns0:cancelCfdiStatusRequestRecord")
                    token = session.token
                    # Asigna valores
                    request.token = token
                    request.uuid = invoice.cfdi_folio_fiscal
                    # Cancela Factura
                    response = client.service.cancelCFDIRequest(request)
                    # Obtiene el codigo que esta en uuid separado por :
                    cod_status = response.uuid.split(":")[1]
                    if response.ok:
                        msg += _(
                            "Please wait 5 minutes before check the invoice in the SAT."
                        )
                        # Invoice canceled, save the cancelation date on database
                        self.pool.get("account.invoice").write(
                            cr,
                            uid,
                            [invoice.id],
                            {
                                "cfdi_fecha_cancelacion": time.strftime(
                                    "%Y-%m-%d %H:%M:%S"
                                )
                            },
                            context=None,
                        )
                        # Cierra sesión
                        client.service.closeSession(token)
                    else:
                        errors = {
                            "200": _("UUID en proceso de cancelación"),
                            "202": _("UUID previamente cancelado"),
                            "203": _("UUID a cancelar no pertenece a contribuyente"),
                            "205": _("UUID a cancelar desconocido"),
                            "206": _("UUID no solicitado para cancelación"),
                            "207": _(
                                "UUID en fecha inválida para cancelación (debe ser cancelado al menos 48 horas depués de su emisión)"
                            ),
                            "208": _("UUID inválido"),
                        }
                        # Cierra sesión
                        client.service.closeSession(token)
                        # error
                        raise osv.except_osv(
                            _("Error"),
                            _("Stamped Code: %s. \n Stamped Message: %s.")
                            % (cod_status, errors[cod_status]),
                        )
                else:
                    raise osv.except_osv(
                        _("Error, could not create a session, code: %d")
                        % str(session.errorCode)
                    )
            except Exception as e:
                # Error al establecer comunicación con el PAC
                logger.debug(str(e))
                raise osv.except_osv(
                    _("Error"), _(u"PAC communication failed. \n%s") % str(e)
                )
        return {"message": msg, "status": cod_status}
