# -*- coding: utf-8 -*-

import re

from openerp import api, fields, models
from openerp.exceptions import ValidationError
from openerp.tools.translate import _


class ResPartnerBank(models.Model):
    _inherit = "res.partner.bank"

    clabe = fields.Char("Clabe Interbancaria", size=64)
    last_acc_number = fields.Char(
        compute="_compute_last_acc_number",
        string="Ultimos 4 digitos",
        size=4,
        store=True,
    )
    currency2_id = fields.Many2one("res.currency", "Currency")
    reference = fields.Char(size=64, help="Reference used in this bank")

    @api.multi
    @api.depends("acc_number")
    def _compute_last_acc_number(self):
        """ Get last 4 digits from account number"""
        for res_bank in self.filtered(lambda r: r.acc_number):
            res_bank.last_acc_number = res_bank.acc_number[-4:]

    @api.constrains("clabe")
    def _check_clabe(self):
        """Ensure field Clabe is valid"""
        regex = re.compile(r"[0-9]{18}")
        for res_bank in self.filtered(lambda r: r.clabe):
            if not regex.match(res_bank.clabe):
                raise ValidationError(_("Invalid Clabe"))
            if res_bank.acc_number and res_bank.acc_number not in res_bank.clabe:
                raise ValidationError(_("Clabe does not correspond to given account"))

    @api.multi
    def is_valid_for_payment_form(self, regex=False):
        """Helper function to ensure current res.partner.bank is suitable for display
        into the account.voucher. Suitable for display means that res.partner.bank
        match regex for given payment_method

        @param regex: regular expression to test account number against
        @type regex: string
        @return: True if current res.partner.bank match regex False otherwise
        """
        self.ensure_one()
        if regex:
            regex = re.compile(regex)
            if regex.match(self.acc_number):
                return True
        return False
