# -*- coding: utf-8 -*-

from openerp.exceptions import except_orm
from openerp.tests.common import TransactionCase


class ResPartnerBank(TransactionCase):
    """Tests for res.partner.bank"""

    def setUp(self):
        super(ResPartnerBank, self).setUp()
        self.res_partner_bank_model = self.env["res.partner.bank"]

    def test_wrong_clabe(self):
        """Raise exception when clabe is incorrect"""
        examples = [
            ("00000000000", "0000000000000000"),  # Invalid clabe
            ("00000001000", "0000000000000000"),  # Clabe not match account
        ]
        for account, clabe in examples:
            with self.assertRaises(except_orm):
                self.env["res.partner.bank"].create(
                    {"state": "bank", "acc_number": account, "clabe": clabe}
                )

    def test_validation_regex(self):
        """Validate several accounts against regex"""
        examples = [
            ("000000000000000000", "[0-9]{11}|[0-9]{18}", True),
            ("00000000000", "[0-9]{11}|[0-9]{18}", True),
            ("000000000", "[0-9]{11}|[0-9]{18}", False),
        ]
        for account, regex, result in examples:
            new_account = self.res_partner_bank_model.create(
                {"state": "bank", "acc_number": account}
            )
            res = new_account.is_valid_for_payment_form(regex)
            self.assertEqual(res, result, "Account not pass validation")
